package com.mortals.xhx.busiz.notify;

import com.alibaba.fastjson.JSON;
import com.aliyuncs.CommonRequest;
import com.aliyuncs.CommonResponse;
import com.aliyuncs.DefaultAcsClient;
import com.aliyuncs.IAcsClient;
import com.aliyuncs.exceptions.ClientException;
import com.aliyuncs.exceptions.ServerException;
import com.aliyuncs.http.MethodType;
import com.aliyuncs.profile.DefaultProfile;
import com.mortals.xhx.busiz.notify.config.SmsConfiguration;
import com.mortals.xhx.common.utils.JacksonUtil;
import lombok.extern.apachecommons.CommonsLog;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.HashMap;
import java.util.Map;

/*
 * 阿里云短信服务
 */
@Component
@CommonsLog
public class AliyunSmsSender implements SmsSender {
    private final String okCode = "OK";
    @Autowired
    private SmsConfiguration smsConfiguration;

    @Override
    public SmsResult send(String phone, String content) {
        SmsResult smsResult = new SmsResult();
        smsResult.setSuccessful(false);
        return smsResult;
    }

    @Override
    public SmsResult sendWithTemplate(String mobile, String templateId, String[] params) {
        SmsResult smsResult = new SmsResult();
        smsResult.setSuccessful(false);
        DefaultProfile profile = DefaultProfile.getProfile(smsConfiguration.getAliyun().getRegionId(), smsConfiguration.getAliyun().getAccessKeyId(), smsConfiguration.getAliyun().getAccessKeySecret());
        IAcsClient client = new DefaultAcsClient(profile);

        CommonRequest request = new CommonRequest();
        request.setSysMethod(MethodType.POST);
        request.setSysDomain("dysmsapi.aliyuncs.com");
        request.setSysVersion("2017-05-25");
        request.setSysAction("SendSms");
        request.putQueryParameter("RegionId", smsConfiguration.getAliyun().getRegionId());
        request.putQueryParameter("PhoneNumbers", mobile);
        request.putQueryParameter("SignName", smsConfiguration.getSign());
        request.putQueryParameter("TemplateCode", templateId);

        String templateParam = "{}";
        if (params.length == 1) {
            Map<String, String> data = new HashMap<>();
            data.put("code", params[0]);
            templateParam = JacksonUtil.toJson(data);
        } else if (params.length > 1) {
            Map<String, String> data = new HashMap<>();
            data.put("code", params[0]);
            for (int i = 1; i < params.length; i++) {
                data.put("code" + i, params[i]);
            }
            templateParam = JacksonUtil.toJson(data);
        }
        request.putQueryParameter("TemplateParam", templateParam);

        try {
            CommonResponse response = client.getCommonResponse(request);
            smsResult.setResult(response.getData());
            String code = JSON.parseObject(response.getData()).getString("Code");

            if (response.getHttpResponse().isSuccess() && okCode.equals(code)) {
                smsResult.setSuccessful(true);
            } else {
                smsResult.setSuccessful(false);
                log.error("短信发送失败：" + response.getData());
            }
            return smsResult;
        } catch (ServerException e) {
            log.error("短信发送异常：", e);
        } catch (ClientException e) {
            log.error("短信发送异常：", e);
        }
        return smsResult;
    }
}
